/**
 * Todo description
 * @moduleName Utils
 */
bento.define('otemoto/utils', [
    'bento/utils',
    'otemoto/utils/merge',
    'otemoto/utils/array',
    'otemoto/utils/random',
    'otemoto/utils/url'
], function (
    Utils,
    Merge,
    ArrayUtils,
    Random,
    Url
) {
    'use strict';
    var _utils = {
        merge: Merge,
        array: ArrayUtils,
        random: new Random(),
        url: Url,
        warnImplementation: function (msg) {
            console.log('not implemented' + (msg ? (': ' + msg) : ('!')));
        }
    };

    Utils.extend(Utils, _utils);
    return Utils;
});
/**
 * @moduleName ScreenAnchor
 * @snippet ScreenAnchor.snippet
ScreenAnchor({
    anchor: new Vector2(0, 0),
    offset: new Vector2(0, 0),
    clamp : new Vector2(-1, -1)
})
 */
bento.define('otemoto/components/screenanchor', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/entity',
    'bento/eventsystem',
    'bento/utils',
    'bento/tween',
    'otemoto/managers/resolution'
], function (
    Bento,
    Vector2,
    Rectangle,
    Entity,
    EventSystem,
    Utils,
    Tween,
    Resolution
) {
    'use strict';
    return function (settings) {
        var viewport = Bento.getViewport();
        var screen = Resolution.getScreen();

        var anchor = settings.anchor || new Vector2(0, 0);
        var offset = settings.offset || new Vector2(0, 0);

        var entity;

        var recalculatePosition = function () {
            var position = new Vector2(0, 0);

            position.x = anchor.x * screen.width;
            position.y = anchor.y * screen.height;

            position.x += offset.x;
            position.y += offset.y;

            if (settings.clamp) {
                var clamp = settings.clamp;
                var safePos = new Vector2(-screen.x, -screen.y);
                safePos.x += viewport.width * anchor.x + offset.x;
                safePos.y += viewport.height * anchor.y + offset.y;
                var dir;

                if (clamp.x >= 0) {
                    var dx = position.x - safePos.x;
                    dir = Utils.sign(dx);
                    dx = Math.abs(dx);
                    position.x = safePos.x + dir * Math.min(dx, clamp.x);
                }

                if (clamp.y >= 0) {
                    var dy = position.y - safePos.y;
                    dir = Utils.sign(dy);
                    dy = Math.abs(dy);
                    position.y = safePos.y + dir * Math.min(dy, clamp.y);
                }
            }

            entity.position = Resolution.screenToViewport(position);
        };

        var component = {
            name: 'screenAnchor',
            onResolutionChanged: function () {
                recalculatePosition()
            },
            start: function (data) {
                Resolution.onChanged(this.onResolutionChanged);
            },
            destroy: function (data) {
                Resolution.offChanged(this.onResolutionChanged);
            },
            attached: function (data) {
                entity = data.entity;
                recalculatePosition();
            }
        };
        return component;
    };
});
/**
 * Resolution Module
 * Provide a desired width and height.
 * The game will scale such that the final resolution is the tightest fit possible around the desired resolution
 */
bento.define('otemoto/managers/resolution', [
    'bento',
    'bento/math/vector2',
    'bento/entity',
    'bento/eventsystem',
    'otemoto/modules/safezoneresize',
    'utils',
    'bento/math/rectangle'
], function (
    Bento,
    Vector2,
    Entity,
    EventSystem,
    SafezoneResize,
    Utils,
    Rectangle
) {
    'use strict';

    var width = 360;
    var height = 640;
    var pixelSize = 2;
    var anchor = new Vector2(0.5, 0.5);

    var safezone = new Rectangle(0, 0, width, height);
    var screen = new Rectangle(0, 0, width, height);

    var viewOffset = new Vector2(0, 0);

    var onChangedListeners = [];

    var margins = {
        left: 0,
        right: 0,
        top: 0,
        bottom: 0
    };

    var viewCorrector = Entity({
        z: 999999,
        name: 'viewCorrector',
        global: true,
        visible: false,
        components: [{
            onPreDraw: function (data) {
                data.renderer.save();
                data.renderer.translate(viewOffset.x, viewOffset.y);
            },
            onPostDraw: function (data) {
                data.renderer.restore();
            },
            start: function () {
                EventSystem.on('preDrawLoop', this.onPreDraw);
                EventSystem.on('postDrawLoop', this.onPostDraw);
            },
            destroy: function () {
                EventSystem.off('preDrawLoop', this.onPreDraw);
                EventSystem.off('postDrawLoop', this.onPostDraw);
            }
        }]
    });

    var updateOffset = function (x, y) {
        viewOffset.x = x;
        viewOffset.y = y;

        var updateInput = function () {
            if (Bento.input) {
                Bento.input.setOffset(viewOffset.scalarMultiply(-1));
            } else {
                Utils.timeout(2, updateInput);
            }
        };
        updateInput();
    };

    var updateCanvas = function () {
        var viewport = Bento.getViewport();

        var canvas = document.getElementById('canvas');

        if (!canvas) {
            return;
        }
        var context = canvas.getContext('2d');

        canvas.width = screen.width * pixelSize;
        canvas.height = screen.height * pixelSize;

        // fit to height
        if (safezone.y !== 0) {
            canvas.style.height = window.innerHeight + 'px';
            canvas.style.width = (viewport.width / viewport.height * window.innerHeight) + 'px';
        } else {
            canvas.style.width = window.innerWidth + 'px';
            canvas.style.height = (viewport.height / viewport.width * window.innerWidth) + 'px';
        }

        if (context.imageSmoothingEnabled) {
            context.imageSmoothingEnabled = false;
        }
        if (context.webkitImageSmoothingEnabled) {
            context.webkitImageSmoothingEnabled = false;
        }
        if (context.mozImageSmoothingEnabled) {
            context.mozImageSmoothingEnabled = false;
        }
        if (context.msImageSmoothingEnabled) {
            context.msImageSmoothingEnabled = false;
        }

        // Bento input updates its viewport before this event, so call this manually
        if (Bento.input) {
            Bento.input.updateCanvas();
        }

        /*
        console.log('canvas resize');
        console.log('canvas');
        console.log(canvas.width, canvas.height);
        console.log('window');
        console.log(window.innerWidth, window.innerHeight);
        console.log('screen');
        console.log(screen.width, screen.height);
        console.log('viewport');
        console.log(viewport.width, viewport.height);
        */
    };

    var updateMargins = function () {
        margins.left = safezone.x;
        margins.top = safezone.y;

        margins.right = screen.width - (safezone.x + safezone.width);
        margins.bottom = screen.height - (safezone.y + safezone.height);
    };

    var onResize = function () {
        var viewport = Bento.getViewport();

        var dimensions = new SafezoneResize({
            width: width,
            height: height,
            anchor: anchor,
            landscape: false
        });

        safezone.width = dimensions.safezone.width;
        safezone.height = dimensions.safezone.height;

        safezone.x = dimensions.safezone.x;
        safezone.y = dimensions.safezone.y;

        screen.width = dimensions.screen.width;
        screen.height = dimensions.screen.height;
        screen.x = -safezone.x;
        screen.y = -safezone.y;

        //This next bit is quite hacky. updateCanvas also updates the input manager, which compares
        //the viewport size to the screen size to get a scale.
        viewport.width = screen.width;
        viewport.height = screen.height;

        updateCanvas();

        viewport.width = safezone.width;
        viewport.height = safezone.height;

        updateOffset(safezone.x, safezone.y);
        updateMargins();

        notifyListeners(onChangedListeners);

        console.log(viewport, screen); /* remove:line */
    };

    var setup = function (settings) {
        width = settings.width || width;
        height = settings.height || height;
        pixelSize = settings.pixelSize || pixelSize;
        anchor = settings.anchor || anchor;

    };
    var init = function () {
        Bento.objects.attach(viewCorrector);

        if (!Utils.isCocoonJs()) {
            window.addEventListener('resize', onResize, false);
            window.addEventListener('orientationchange', onResize, false);
        }
        onResize();
    };

    var viewportToScreen = function (vec) {
        vec.x += viewOffset.x;
        vec.y += viewOffset.y;
        return vec;
    };

    var screenToViewport = function (vec) {
        vec.x -= viewOffset.x;
        vec.y -= viewOffset.y;
        return vec;
    };

    var notifyListeners = function (listeners) {
        var ii;
        for (ii = listeners.length - 1; ii >= 0; ii--) {
            listeners[ii]();
        }
    };

    //use with addListener.bind(_array_)
    var addListener = function (callback) {
        var idx = this.indexOf(callback);
        if (idx !== -1) {
            return;
        }
        this.push(callback);
    };

    var removeListener = function (callback) {
        var idx = this.indexOf(callback);
        if (idx === -1) {
            return;
        }
        this.splice(idx, 1);
    };

    var setSafeZone = function (w, h) {
        if (width === w && height === h) {
            return;
        }

        width = w;
        height = h;

        onResize();
    };

    var module = {
        setup: setup,
        init: init,
        update: function () {
            onResize();
        },
        getPixelSize: function () {
            return pixelSize;
        },
        getMargins: function () {
            return margins;
        },
        getScreen: function () {
            return screen;
        },
        setSafeZone: setSafeZone,
        viewportToScreen: viewportToScreen,
        screenToViewport: screenToViewport,
        onChanged: addListener.bind(onChangedListeners),
        offChanged: removeListener.bind(onChangedListeners)
    };

    return module;
});
/**
 * @moduleName SafezoneResize
 * Similar to the AutoResize, this module aids in rescaling the canvas. The
 * difference is that this resizer fits an area of the desired size as snugly as
 * possible in the center of the screen
 *
 * <br>Exports: Constructor
 * @module bento/safezoneresize
 * @moduleName SafezoneResize
 * @param {Object} settings - Required
 * @param {Number} settings.width - Width of the safezone
 * @param {Number} settings.height - Height of the safezone
 * @param {Boolena} settings.landscape - Wether the device orientation should be landscape or not
 * @param {Vector2} settings.anchor - Where to anchor the safezone in the screen. Default is centered (0.5, 0.5)
 * @returns {Object} Object with two properties: screen and safezone dimensions (Rectangles)
 */
bento.define('otemoto/modules/safezoneresize', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle'
], function (
    Bento,
    Vector2,
    Rectangle
) {
    'use strict';
    return function (settings) {
        var screen = new Rectangle(0, 0, settings.width, settings.height);
        var safezone = new Rectangle(0, 0, settings.width, settings.height);

        var anchor = settings.anchor || new Vector2(0.5, 0.5);

        var innerWidth = window.innerWidth;
        var innerHeight = window.innerHeight;

        var devicePixelRatio = window.devicePixelRatio;

        var deviceWidth = innerWidth * devicePixelRatio;
        var deviceHeight = innerHeight * devicePixelRatio;

        if (settings.landscape) {
            (function () {
                var tmp = deviceWidth;
                deviceWidth = deviceHeight;
                deviceHeight = tmp;
            })();
        }

        var screenRatio = deviceWidth / deviceHeight;
        var desiredRatio = settings.width / settings.height;

        if (screenRatio > 1) {
            //user is holding device wrong
            screenRatio = 1 / screenRatio;
        }

        if (screenRatio > desiredRatio) {
            //screen is too wide, clip to height
            screen.width = settings.height * screenRatio;
        } else {
            //screen is too tall, clip to width
            screen.height = settings.width / screenRatio;
        }

        safezone.x = (screen.width - settings.width) * anchor.x;
        safezone.y = (screen.height - settings.height) * anchor.y;

        return {
            screen: screen,
            safezone: safezone
        };
    };
});
/**
 * Style Manager
 * TODO: documentation
 * @moduleName StyleManager
 * @snippet StyleManager.snippet
StyleManager({

})
 */
bento.define('otemoto/modules/stylemanager', [

], function (

) {
    'use strict';
    var _setProperty = function (prop, value, node) {
        node[prop] = value;
    };

    var _applyTo = function (style, node) {
        var key;
        for (key in style) {
            _setProperty(key, style[key], node);
        }
    };

    return function () {
        var defaults = {};
        var styles = {};

        var setDefaults = function (style) {
            _applyTo(style, defaults);
        };

        var setStyle = function (key, style) {
            styles[key] = styles[key] || {};
            _applyTo(style, styles[key]);
        };

        var getProperty = function (propertyKey, styleKey, override) {
            if (override && override[propertyKey] !== undefined) return override[propertyKey];

            var style = styles[styleKey]
            if (style) {
                if (style[propertyKey] !== undefined) return style[propertyKey];

                var next = style;
                while (next.base) {
                    next = styles[next.base];
                    if (!next) {
                        Console.warn('dead end in style dependencies: ' + styleKey);
                        break;
                    }
                    //TODO: this only detects a loop if the dependencies loop back to the starting style
                    if (next === style) {
                        Console.warn('circular loop in style dependencies: ' + styleKey);
                        break;
                    }
                    if (next[propertyKey] !== undefined) {
                        //TODO, should this actually happen? The idea is you just define styles once and not modify them afterwards
                        //so, this allows for a quicker look up, but 'breaks' the dependency model
                        _setProperty(propertyKey, next[propertyKey], style);
                        return style[propertyKey];
                    }
                }
            }

            return defaults[propertyKey];
        };

        /**
         * Returns the whole style object. Parses all the bases at first
         * @return {[object]} parsed style object
         */
        var getStyle = function (styleKey) {
            var result = {};
            var style = styles[styleKey]
            var current = style;
            var key;
            while (current) {
                for (key in current) {
                    result[key] = result[key] || current[key];
                    style[key] = result[key];
                }
                if (current.base) {
                    current = styles[current.base];
                } else {
                    current = undefined;
                }
            }
            return result;
        };

        var module = {
            setDefaults: setDefaults,
            setStyle: setStyle,
            getProperty: getProperty,
            getStyle: getStyle
        };
        return module;
    };
});
/**
 * Dummy Entity for layout designs
 * @moduleName Dummy
 * @snippet Dummy.snippet
Dummy(${1:16}, ${2:16}, ${3:false});
 */
bento.define('otemoto/ui/dummy', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/entity',
    'bento/utils'
], function (
    Bento,
    Vector2,
    Rectangle,
    Entity,
    Utils
) {
    'use strict';
    return function (width, height, debug) {
        width = (width === undefined) ? 16 : width;
        height = (height === undefined) ? 16 : height;

        var dummy = new Entity({
            name: 'dummy',
            visible: false,
            dimension: new Rectangle(0, 0, width, height),
            components: []
        });

        if (debug) {
            dummy.visible = true;
            dummy.attach({
                draw: function (data) {
                    data.renderer.strokeRect([1, 1, 0, 1], 0, 0, width, height, 1);
                }
            });
        }

        return dummy;
    };
});
/**
 * Create a frame around & behind a provided child
 * @moduleName Frame
 * @snippet Frame.snippet
Frame({
    content : ${1},
    imageName : ${2:'frames/'},
    marginHor : ${3:4},
    marginVer : ${3:4}
})
 */
bento.define('otemoto/ui/frame', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'bento/gui/text',
    'bento/utils',
    'bento/tween',
    'bento/components/nineslice'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    ClickButton,
    Counter,
    Text,
    Utils,
    Tween,
    NineSlice
) {
    'use strict';
    return function (settings) {
        var img = Bento.assets.getImage(settings.imageName);
        var frameCountX = settings.frameCountX || 1;
        var frameCountY = settings.frameCountY || 1;

        var sliceWidth = (img.width / frameCountX) / 3;
        var sliceHeight = (img.height / frameCountY) / 3;

        var marginHor = Utils.getDefault(settings.marginHor, sliceWidth);
        var marginVer = Utils.getDefault(settings.marginVer, sliceHeight);

        var content = settings.content;

        var nineSlice = new NineSlice({
            imageName: settings.imageName,
            origin: new Vector2(-content.dimension.x + marginHor, -content.dimension.y + marginVer),
            width: content.dimension.width + marginHor * 2,
            height: content.dimension.height + marginVer * 2
        });

        var entity = new Entity({
            z: settings.z || 0,
            position: settings.position || new Vector2(0, 0),
            updateWhenPaused: 0,
            float: false,
            name: 'frame',
            components: [
                nineSlice,
                content
            ]
        });
        return entity;
    };
});
/**
 * A wrapper around Bento's text object, with some QoL improvements. You can set universal styles, and simply provide
 * the [style] property. Any specific properties can still easily be overriden. The order is as follows
 * Defined properties > properties derived from style > defaults
 *
 * The label requires fixed dimensions, and instead of passing [align] and [baseline] onto the main Text entity, it uses
 * these to position the text within the bounding box of the label.
 *
 * A value for the text can either be provided trough [value] or [key]. If [key] is used, the text value will be requested from a dictionary
 *  !Attention, at the moment, this is hardcoded to be through a module @ modules/localization, that has a .getText() function
 *
 * an object with replacements can also be provided in the format {key : 'replacement'}. the Label entity will search the displayed text for instances of $KEY$ and replace it with the provided value
 *
 * A new [yCorrection] property is provided. Since not all fonts have their baselines set up correctly, fiddling with yCorrection (usually between -0.5 and 0.5) should ensure that all baseline alignments actually line up as expected
 *
 * TODO:
 *     -Allow for a custom dictionary lookup function, detangle from default bento project
 *     -Allow to set a custom key replacement function
 *     -Add new properties that were added in Bento's default text module
 *         -Gradient, shadows
 *
 * @moduleName Label
 * @snippet Label.constructor
Label({
    z: 0,
    style : 'default',
    dimension : {
        x : 0,
        y : 0,
        width : 120,
        height : 36
    },
    key : 'lbl_text'
})
 */
bento.define('otemoto/ui/label', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/utils',
    'bento/gui/text',
    'modules/localization',
    'otemoto/modules/stylemanager',
    'bento/entity'
], function (
    Bento,
    Vector2,
    Rectangle,
    Utils,
    Text,
    Localization,
    StyleManager,
    Entity
) {
    'use strict';

    var LORUM_IPSUM = "Now, this is a story all about how my life got flipped upside down. And I'd like to take a minute - just sit right here - I'll tell you how I became the prince of a town called Bel Air. In west Philadelphia born and raised, on the playground was where I spent most of my days. Chillin' out, maxin', relaxin' all cool, and all shootin' B-ball outside of school. When a couple of guys who were up to no good started making trouble in the neighbourhood. I got in one little fight and my mom got scared. She said, \"You're movin' with your auntie and uncle in Bel Air\"";

    var DEFAULT_STYLE = {
        font: 'font',
        fontSize: 16,
        fontColor: '#ffffff',
        align: 'left',
        baseline: 'bottom',
        ySpacing: 0,
        lineWidth: 0,
        linebreaks: true,
        strokeStyle: '#000000',
        yCorrection: 0,
        androidCorrection: 0,
        fontScale: 1,
        marginX: 8,
        marginY: 8,
        upperCase: false,
        lowerCase: false
    };

    var _styles = new StyleManager({});
    _styles.setDefaults(DEFAULT_STYLE);

    var Label = function (settings) {
        var textValue;


        var margin = new Vector2(_styles.getProperty('marginX', settings.style, settings), _styles.getProperty('marginY', settings.style, settings));

        var position = new Vector2(0, 0);
        var dimension;
        var maxWidth;
        var maxHeight;
        var anchor;

        var replacements = settings.replacements || {};


        var getTextValue = function () {
            if (settings.key) {
                textValue = Localization.getText(settings.key);
            } else {
                textValue = settings.value || "";
            }

            if (settings.lorumIpsum) {
                textValue = "";
                while (settings.lorumIpsum > 0) {
                    var count = Math.min(LORUM_IPSUM.length, settings.lorumIpsum);
                    textValue += LORUM_IPSUM.substr(0, count);
                    settings.lorumIpsum -= count;
                }
            }

            textValue = '' + textValue; //parse numbers as string

            if (_styles.getProperty('upperCase', settings.style, settings)) {
                textValue = textValue.toUpperCase();
            } else if (_styles.getProperty('lowerCase', settings.style, settings)) {
                textValue = textValue.toLowerCase();
            }
        };

        var getDimensions = function () {
            var align = _styles.getProperty('align', settings.style, settings);
            var baseline = _styles.getProperty('baseline', settings.style, settings);

            position.x = settings.dimension.x;
            position.y = settings.dimension.y;

            maxWidth = settings.dimension.width;
            maxHeight = settings.dimension.height;

            if (settings.position) {
                position = settings.position;
            }

            anchor = settings.anchor || new Vector2(0, 0);

            if (align) {
                switch (align) {
                    case 'left':
                        anchor.x = 0;
                        break;
                    case 'center':
                        anchor.x = 0.5;
                        break;
                    case 'right':
                        anchor.x = 1;
                        break;
                }
            }

            if (baseline) {
                switch (baseline) {
                    case 'top':
                        anchor.y = 0;
                        break;
                    case 'middle':
                        anchor.y = 0.5;
                        break;
                    case 'bottom':
                        anchor.y = 1;
                        break;
                }
            }

            dimension = new Rectangle(-anchor.x * settings.dimension.width, -anchor.y * settings.dimension.height, settings.dimension.width, settings.dimension.height);
        };

        var realign = function () {
            var yCorrection = _styles.getProperty('yCorrection', settings.style, settings);

            //y correction (because most fonts just mess up the baseline)
            var ycc = yCorrection * (1 - anchor.y);
            text.position.y = ycc * text.getEffectiveFontSize();

            //strokes seem to offset the text a tad?
            text.position.y -= _styles.getProperty('lineWidth', settings.style, settings);

            //android offset
            //cocoon pls
            if (Utils.isAndroidDevice() && Utils.isCocoonJs()) {
                var androidCorrection = _styles.getProperty('androidCorrection', settings.style, settings);
                //console.log('android correction', androidCorrection);
                text.position.y += androidCorrection;
            }
        };

        getTextValue();
        getDimensions();

        var text = new Text({
            drawDebug: settings.debug,
            text: textValue,
            font: _styles.getProperty('font', settings.style, settings),
            align: _styles.getProperty('align', settings.style, settings),
            textBaseline: _styles.getProperty('baseline', settings.style, settings),
            margin: margin,
            fontSize: Math.round(_styles.getProperty('fontSize', settings.style, settings) * _styles.getProperty('fontScale', settings.style, settings)),
            fontColor: _styles.getProperty('fontColor', settings.style, settings),
            maxWidth: maxWidth,
            maxHeight: maxHeight,
            ySpacing: _styles.getProperty('ySpacing', settings.style, settings),
            linebreaks: _styles.getProperty('linebreaks', settings.style, settings),
            lineWidth: _styles.getProperty('lineWidth', settings.style, settings),
            strokeStyle: _styles.getProperty('strokeStyle', settings.style, settings),
            pixelStroke: true
        });

        realign();

        var container = new Entity({
            name: 'label',
            position: position,
            z: settings.z || 0,
            components: [
                text
            ]
        });

        container.extend({
            setKey: function (k) {
                //TODO make key function replaceable (not every game might use a Localization module)
                container.setValue(Localization.getText(k));
            },
            setValue: function (val, force, style) {
                var key;
                var re;
                val = '' + val; //parse numbers as string
                //TODO make the replacement function customizable
                for (key in replacements) {
                    re = new RegExp('\\$' + key.toUpperCase() + '\\$', 'g');
                    val = val.replace(re, replacements[key]);
                }

                if (_styles.getProperty('upperCase', settings.style, settings)) {
                    val = val.toUpperCase();
                } else if (_styles.getProperty('lowerCase', settings.style, settings)) {
                    val = val.toLowerCase();
                }

                if (textValue === val && !force) {
                    return;
                }
                textValue = val;

                text.setText(textValue, style);

                realign();
            },
            setReplacement: function (key, value) {
                replacements[key] = value;
                container.setValue(textValue, true);
            },
            changeStyle: function (newStyle) {
                if (typeof newStyle === 'string') {
                    newStyle = _styles.getStyle(newStyle);
                } else if (newStyle.base) {
                    newStyle = Utils.merge(newStyle, _styles.getStyle(newStyle.base));
                }
                container.setValue(textValue, true, newStyle);
            },
            getValue: function () {
                return textValue;
            },
            getText: function () {
                return text;
            }
        });


        if (dimension) {
            container.dimension = dimension;
        }

        if (settings.debug) {
            container.attach({
                draw: function (data) {
                    var dim = data.entity.dimension;
                    data.renderer.fillRect([1, 0, 0.5, 0.33], dim.x, dim.y, dim.width, dim.height);

                    data.renderer.strokeCircle([0, 1, 0.5, 1], 0, 0, 3, 0, Math.PI * 2, 1);
                }
            });
        }

        return container;
    };

    Label.setStyle = function (key, definition) {
        _styles.setStyle(key, definition);
    };

    return Label;
});
/**
 * Button with a label. Provide button properties and label properties.
 *
 * If the button is nineSliced (default true), the innerWidth and innerHeight will match the dimensions of the label
 *
 * TODO:
 *     -more extensive documentation
 *         -buttonStyles
 *         -labelYOffsets,
 *         -fontStyle
 *
 * @moduleName LabelButton
 * @snippet LabelButton.constructor
LabelButton({
    position: new Vector2(0, 0),
    style : 'default',
    z: 0,
    onClick: function(){},
    button: {
        updateWhenPaused: 0,
        float: false
    },
    // beautify preserve:start
    label: [
        {direction: 'x'},
        {type: 'text', key: 'btn_x', width : 120, height : 36}
    ]
    // beautify preserve:end
})
 */
bento.define('otemoto/ui/labelbutton', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'bento/gui/text',
    'bento/utils',
    'bento/tween',
    'ui/layoutbuilder',
    'ui/label',
    'otemoto/modules/stylemanager'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    ClickButton,
    Counter,
    Text,
    Utils,
    Tween,
    LayoutBuilder,
    Label,
    StyleManager
) {
    'use strict';
    var DEFAULT_STYLE = {
        fontStyle: 'default',
        imageName: '',
        labelYOffsets: [0, 0],
        nineSliced: true,
        sfx: '',
        frameCountX: 1,
        frameCountY: 2,
        marginHor: 0,
        marginVer: 0
    };

    var _styles = new StyleManager({});

    _styles.setDefaults(DEFAULT_STYLE);

    var LabelButton = function (settings) {
        var labelLayout;
        var label = settings.label || [];

        var nineSliced = _styles.getProperty('nineSliced', settings.style, settings);
        var labelYOffsets = _styles.getProperty('labelYOffsets', settings.style, settings);

        var imageName = _styles.getProperty('imageName', settings.style, settings.button);
        var marginHor = _styles.getProperty('marginHor', settings.style, settings.button);
        var marginVer = _styles.getProperty('marginVer', settings.style, settings.button);

        var textLabels = [];
        var images = [];

        var activeFontStyle = 'fontStyle';

        var parseAndBuildLabel = function () {
            var parseText = function (entry) {
                //default text settings
                var lblSettings = {
                    style: _styles.getProperty('fontStyle', settings.style, settings),
                    align: 'center',
                    baseline: 'middle',
                    dimension: {
                        x: 0,
                        y: 0,
                        width: entry.width,
                        height: entry.height
                    }
                };
                Utils.extend(lblSettings, entry);

                var label = new Label(lblSettings);
                textLabels.push(label);
                return label;
            };

            var parseImage = function (entry) {
                var fcx = entry.frameCountX || 1;
                var fcy = entry.frameCountY || 1;

                var anims = {
                    'up': {
                        frames: [0]
                    },
                    'down': {
                        frames: [1]
                    },
                    'inactive': {
                        frames: [2]
                    }
                };
                var count = fcx * fcy;
                if (count < 3) {
                    anims['inactive'].frames = [1];
                }
                if (count < 2) {
                    delete anims['inactive'];
                    delete anims['down'];
                }

                entry.animations = entry.animations || anims;
                if (entry.animations['default']) {
                    entry.animations['up'] = entry.animations['default'];
                }

                var img = new Entity({
                    components: [
                        new Sprite(entry)
                    ]
                });
                images.push(img.getComponent('sprite'));
                return img;
            };

            var parseEntry = function (entry) {
                var ii;
                var current;
                for (ii = 1; ii < entry.length; ii++) {
                    current = entry[ii];
                    if (Array.isArray(current)) {
                        parseEntry(current);
                    } else if (current.type === 'text') {
                        entry[ii] = parseText(current);
                    } else if (current.type === 'image') {
                        entry[ii] = parseImage(current);
                    }
                }
            };

            label = Utils.array.copy(label);

            parseEntry(label);

            labelLayout = LayoutBuilder.build(label);
        };

        if (settings.label) {
            parseAndBuildLabel();
        }

        var container = new Entity({});

        var adjustLabel = function (idx, font, anim) {
            if (labelYOffsets && labelYOffsets[idx] !== undefined) {
                container.transform.y = labelYOffsets[idx];
            }

            if (font !== activeFontStyle) {
                var fontStyle = _styles.getProperty(font, settings.style, {});
                if (fontStyle) {
                    activeFontStyle = font;
                    Utils.forEach(textLabels, function (lbl, i, l, breakLoop) {
                        lbl.changeStyle(fontStyle);
                    });
                }
            }

            Utils.forEach(images, function (img, i, l, breakLoop) {
                var anims = img.animations;
                if (anims[anim]) {
                    img.setAnimation(anim);
                }
            });
        };

        var onButtonDown = function () {
            settings.onButtonDown && settings.onButtonDown();
            adjustLabel(1, 'downFontStyle', 'down');
        };

        var onButtonUp = function () {
            settings.onButtonUp && settings.onButtonUp();
            adjustLabel(0, 'fontStyle', 'up');
        };

        var onActivate = function () {
            adjustLabel(0, 'fontStyle', 'up');
        };

        var onDeactivate = function () {
            adjustLabel(2, 'downFontStyle', 'inactive');
            adjustLabel(2, 'inactiveFontStyle', 'inactive');
        };

        var addToContainer = function (e, dontParse) {
            container.attach(e);

            if (dontParse) {
                return;
            }

            e.getComponent('sprite', function (sprite) {
                images.push(sprite);
            });

            e.getComponent('label', function (label) {
                textLabels.push(label);
            });
        };

        var removeFromContainer = function (e) {
            Utils.warnImplementation('parse sprites & labels');
            container.remove(e);
        };

        var btnSettings = {
            z: settings.z || 0,
            name: settings.name || 'button',
            sfx: _styles.getProperty('sfx', settings.style, settings.button),
            imageName: imageName,
            frameCountX: _styles.getProperty('frameCountX', settings.style, settings.button),
            frameCountY: _styles.getProperty('frameCountY', settings.style, settings.button),
            animations: _styles.getProperty('animations', settings.style, settings.button),
            position: settings.position || new Vector2(0, 0),
            updateWhenPaused: 0,
            float: false,
            onClick: settings.onClick,
            onButtonDown: onButtonDown,
            onButtonUp: onButtonUp
        };

        Utils.extend(btnSettings, settings.button || {});

        //dynamic width and height
        if (nineSliced && labelLayout) {
            (function () {
                var sliceImg = Bento.assets.getImage(imageName);
                var sliceWidth = marginHor || (sliceImg.width / btnSettings.frameCountX) / 3;
                var sliceHeight = marginVer || (sliceImg.height / btnSettings.frameCountY) / 3;

                var marginH = marginHor || sliceWidth;
                var marginV = marginVer || sliceHeight;

                var totalWidth = settings.width || Math.max(2 * sliceWidth, labelLayout.dimension.width + marginH * 2);
                var totalHeight = settings.height || Math.max(2 * sliceHeight, labelLayout.dimension.height + marginV * 2);

                btnSettings.nineSliceSettings = {
                    width: totalWidth,
                    height: totalHeight
                };
            })();
        } else if (nineSliced && settings.width) {
            btnSettings.nineSliceSettings = {
                width: settings.width,
                height: settings.height
            };
        }

        var baseButton = new ClickButton(btnSettings);

        if (labelLayout) {
            labelLayout.position = settings.labelOffset || new Vector2(0, 0);
            container.attach(labelLayout);
        }
        baseButton.attach(container);
        container.dimension = baseButton.dimension.clone();

        baseButton.extend({
            setLabel: function (l) {
                if (labelLayout) {
                    container.remove(labelLayout);
                }
                label = l;
                parseAndBuildLabel();
                container.attach(labelLayout);
            },
            addToContainer: addToContainer,
            removeFromContainer: removeFromContainer,
            activate: function () {
                baseButton.setActive(true);
                onActivate();
            },
            deactivate: function () {
                baseButton.setActive(false);
                onDeactivate();
            }
        });

        return baseButton;
    };

    LabelButton.setStyle = function (key, definition) {
        _styles.setStyle(key, definition);
    };

    return LabelButton;
});
/**
 * Layout. Add children, they will be automatically laid out
 * @moduleName Layout
 * @snippet Layout.snippet
Layout({
    position: new Vector2(0, 0)
})
 */
bento.define('otemoto/ui/layout', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'bento/gui/text',
    'bento/utils',
    'bento/tween'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    ClickButton,
    Counter,
    Text,
    Utils,
    Tween
) {
    'use strict';

    var ALT_DIR = {
        'x': 'y',
        'y': 'x'
    };

    var ALT_DIM = {
        'width': 'height',
        'height': 'width'
    };

    return function (settings) {
        var viewport = Bento.getViewport();
        var position = settings.position || new Vector2(0, 0);

        var margin = (settings.margin === undefined) ? 10 : settings.margin;

        var align = (settings.align === undefined) ? 0.5 : settings.align;
        var anchor = settings.anchor || new Vector2(0.5, 0.5);

        var dir = settings.direction || 'y';
        var dim = (dir === 'x') ? 'width' : 'height';
        var altDir = ALT_DIR[dir];
        var altDim = ALT_DIM[dim];

        var children = [];

        var container = new Entity({
            // components: [{
            //     draw: function (data) {
            //         data.renderer.fillRect([0, 1, 0, 0.5], 0, 0, entity.dimension.width, entity.dimension.height);
            //     }
            // }]
            name: 'layoutContainer'
        });

        var recalculate = function () {
            var max = {
                'width': 0,
                'height': 0
            };

            var currentPos = 0;
            var ii;
            var child;

            for (ii = 0; ii < children.length; ii++) {
                child = children[ii];

                max['width'] = Math.max(max['width'], child.dimension.width);
                max['height'] = Math.max(max['height'], child.dimension.height);

                child.position[dir] = currentPos - child.dimension[dir];

                currentPos += child.dimension[dim] + margin;
            }
            currentPos -= margin;

            entity.dimension[dim] = currentPos;
            entity.dimension[altDim] = max[altDim];

            container.position.x = -entity.dimension.width * anchor.x;
            container.position.y = -entity.dimension.height * anchor.y;

            entity.dimension.x = -entity.dimension.width * anchor.x;
            entity.dimension.y = -entity.dimension.height * anchor.y;

            for (ii = 0; ii < children.length; ii++) {
                child = children[ii];

                var ww = child.dimension[altDim];
                var xx = child.dimension[altDir];
                var ll = ww / 2 - xx;

                child.position[altDir] = -xx + (max[altDim] - ww) * align;
            }
        };

        var addChild = function (c) {
            var idx = children.indexOf(c);
            if (idx !== -1) {
                return;
            }
            children.push(c);
            container.attach(c);

            recalculate();
        };

        var removeChild = function (c) {
            var idx = children.indexOf(c);
            if (idx === -1) {
                return;
            }
            children.splice(idx, 1);
            container.remove(c);

            recalculate();
        };

        var entity = new Entity({
            z: settings.z || 0,
            name: 'layout',
            family: ['layout', 'ui'],
            position: position,
            updateWhenPaused: Bento.objects.isPaused(),
            float: false,
            components: [
                container
            ]
        }).extend({
            addChild: addChild,
            add: addChild,
            removeChild: removeChild,
            remove: removeChild,
            recalculate: recalculate,
            getChildAt: function (idx) {
                return children[idx];
            }
        });

        if (settings.debug) {
            entity.attach({
                draw: function (data) {
                    data.renderer.strokeRect(
                        [1, 0, 0, 1],
                        data.entity.dimension.x, data.entity.dimension.y,
                        data.entity.dimension.width, data.entity.dimension.height,
                        2);
                }
            });
        }

        return entity;
    };
});
/**
 * @moduleName LayoutBuilder
 */
bento.define('otemoto/ui/layoutbuilder', [
    'bento',
    'bento/eventsystem',
    'utils',
    'ui/layout'
], function (
    Bento,
    EventSystem,
    Utils,
    Layout
) {
    'use strict';
    var parseNode = function (node) {
        var props = node[0];
        var idx = 1;

        var layout = new Layout(props);
        var current;
        while (idx < node.length) {
            current = node[idx];

            if (Array.isArray(current)) {
                layout.add(parseNode(current))
            } else if (current) {
                layout.add(current)
            }
            idx++;
        }
        layout.recalculate();

        return layout;
    };

    var build = function (base) {
        return parseNode(base);
    };

    var module = {
        build: build
    };
    return module;
});
/**
 * ToggleButton
 * @moduleName ToggleButton
 * @snippet ToggleButton.snippet
ToggleButton({
    position: new Vector2(0, 0)
})
 */
bento.define('otemoto/ui/togglebutton', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'bento/gui/text',
    'bento/utils',
    'bento/tween',
    'ui/labelbutton'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    ClickButton,
    Counter,
    Text,
    Utils,
    Tween,
    LabelButton
) {
    'use strict';
    return function (settings) {
        var labels = settings.labels;
        var idx = settings.idx || 0;

        var onClick = function () {
            idx = (idx + 1) % labels.length;
            button.setLabel(labels[idx]);
            settings.onToggle && settings.onToggle(idx);
        };

        var properties = {
            onClick: onClick
        };

        Utils.extend(settings, properties);

        var button = new LabelButton(settings);

        button.setLabel(labels[idx]);

        return button;
    };
});
/**
 * Array helper functions
 * @moduleName ArrayUtils
 */
bento.define('otemoto/utils/array', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/entity',
    'bento/eventsystem',
    'bento/utils',
    'bento/tween'
], function (
    Bento,
    Vector2,
    Rectangle,
    Entity,
    EventSystem,
    Utils,
    Tween
) {
    'use strict';

    var clone = function (array) {
        return array.concat([]);
    };

    var shuffle = function (array, randObject) {
        var idx;
        var tmp;
        var ii;

        var rand = Math.random;
        if (randObject) {
            rand = randObject.random;
        }

        for (ii = array.length; ii > 0; ii--) {
            idx = Math.floor(rand() * ii);
            tmp = array[idx];
            array[idx] = array[ii - 1];
            array[ii - 1] = tmp;
        }

        return array;
    };

    var copy = function (array) {
        var newArray = [];

        var node;
        var ii;

        for (ii = 0; ii < array.length; ii++) {
            node = array[ii];
            if (Array.isArray(node)) {
                newArray[ii] = copy(node);
            } else {
                newArray[ii] = array[ii];
            }
        }

        return newArray;
    };

    var stepped = function (begin, interval, end) {
        var current = begin;
        var count = Math.floor((end - begin) / interval);
        var arr = [];

        var ii;
        for (ii = 0; ii <= count; ii++) {
            arr.push(current);
            current += interval;
        }
        return arr;
    };

    var create2D = function (width, height, def) {
        var ii;
        var jj;
        var arr = [];
        for (ii = 0; ii < height; ii++) {
            arr[ii] = [];
            for (jj = 0; jj < width; jj++) {
                arr[ii][jj] = def;
            }
        };

        return arr;
    };

    var create3D = function (width, height, depth, def) {
        var ii;
        var jj;
        var kk;
        var arr = [];
        for (ii = 0; ii < height; ii++) {
            arr[ii] = [];
            for (jj = 0; jj < width; jj++) {
                arr[ii][jj] = [];
                for (kk = 0; kk < depth; kk++) {
                    arr[ii][jj][kk] = def;
                }
            }
        };

        return arr;
    };

    var mod = {
        clone: clone,
        shuffle: shuffle,
        copy: copy,
        stepped: stepped,
        create2D: create2D,
        create3D: create3D

    };

    return mod;
});
/**
 * JsonFinder
 * Helps find all json files in a folder. Works with 'packed' jsons as well
 * @moduleName JsonFinder
 * @snippet JsonFinder.snippet
JsonFinder.find('base');
 */
bento.define('otemoto/utils/jsonfinder', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'bento/gui/text',
    'bento/utils',
    'bento/tween'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    ClickButton,
    Counter,
    Text,
    Utils,
    Tween
) {
    'use strict';
    var find = function (base) {
        var result = [];
        var key;

        var allJsons = Bento.assets.getAssets().json;

        for (key in allJsons) {
            if (key.indexOf(base) !== 0) continue;
            var entry = {};

            entry.path = key;
            entry.base = base;
            entry.data = Bento.assets.getJson(key);
            entry.key = key.replace(base, '');

            result.push(entry);
        }

        return result;
    };

    return {
        find: find
    };
});
/**
 * Combines two objects into a third object (not modifying the originals)
 * This is a deep copy, so do not use on large dynamic objects!
 *
 * the first object has priority over the second object
 * @moduleName Merge
 */
bento.define('otemoto/utils/merge', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/entity',
    'bento/eventsystem',
    'bento/utils',
    'bento/tween'
], function (
    Bento,
    Vector2,
    Rectangle,
    Entity,
    EventSystem,
    Utils,
    Tween
) {
    'use strict';
    var isObjLiteral = function (_obj) {
        var _test = _obj;
        return (typeof _obj !== 'object' || _obj === null ?
            false :
            (
                (function () {
                    while (!false) {
                        if (Object.getPrototypeOf(_test = Object.getPrototypeOf(_test)) === null) {
                            break;
                        }
                    }
                    return Object.getPrototypeOf(_obj) === _test;
                })()
            )
        );
    };

    return function (obj1, obj2) {
        var prop;
        var result = {};
        for (prop in obj1) {
            if (obj1.hasOwnProperty(prop)) {
                if (isObjLiteral(obj1[prop])) {
                    result[prop] = Utils.merge(obj1[prop], obj2[prop] || {});
                } else {
                    result[prop] = obj1[prop];
                }
            }
        }
        for (prop in obj2) {
            if (obj2.hasOwnProperty(prop)) {
                if (result.hasOwnProperty(prop)) {
                    continue;
                }
                if (isObjLiteral(obj2[prop])) {
                    result[prop] = Utils.merge({}, obj2[prop]);
                } else {
                    result[prop] = obj2[prop];
                }
            }
        }
        return result;
    };
});
/**
 * Player Data
 * Wrapper for bento save state manager
 *
 * Will only save data if the keys exist
 * Every key has default data
 * @moduleName PlayerData
 *
 * TODO. what if the defined key goes to a deeper level than request. e.g.
 *		DEF = 'LEVEL/* /{SCORE, TIME, etc...}'
 * What should PlayerData.load('LEVEL/2') return? Null, or an object with all the subkeys & values in it?
 */
bento.define('otemoto/utils/playerdata', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/entity',
    'bento/eventsystem',
    'bento/utils',
    'bento/tween'
], function (
    Bento,
    Vector2,
    Rectangle,
    Entity,
    EventSystem,
    Utils,
    Tween
) {
    'use strict';
    var KEYS = {};
    var cache = {};

    var getDefaultValue = function (key) {
        var sections = key.split('/');
        var currentNode = KEYS;
        var currentKey = sections.shift();
        var progress;

        while (true) {
            if (currentNode.hasOwnProperty(currentKey)) {
                currentNode = currentNode[currentKey];
                progress = true;
            } else if (currentNode.hasOwnProperty('*')) {
                currentNode = currentNode['*'];
                progress = true;
            } else {
                return undefined;
            }

            if (sections.length === 0) {
                //found it!
                return currentNode;
            }
            currentKey = sections.shift();
        }
    };

    var storeInCache = function (key, value) {
        cache[key] = value;
    };

    var loadFromCache = function (key) {
        return cache[key];
    };

    var save = function (key, value) {
        if (getDefaultValue(key) === undefined) {
            Utils.log('illegal save key ' + key);
            return;
        }
        if (value === undefined) {
            Utils.log('no save value provided for ' + key);
            return;
        };

        storeInCache(key, value);
        return Bento.saveState.save(key, value);
    };

    var load = function (key) {
        var def = getDefaultValue(key);
        if (def === undefined) {
            Utils.log('illegal save key ' + key);
            return;
        }

        var val = loadFromCache(key);
        if (!val) {
            val = Bento.saveState.load(key, def);
            storeInCache(key, val);
        }
        return val;
    };

    var addKeys = function (keys) {
        Bento.saveState.saveKeys = true;

        KEYS = Utils.merge(KEYS, keys);
    };

    var module = {
        save: save,
        load: load,
        addKeys: addKeys
    };
    return module;
});
/**
 * Seeded random
 */
bento.define('otemoto/utils/random', [

], function (

) {
    'use strict';
    var M = 0xffffffff; //modulus - 2^32 or 4 bytes
    var A = 1664525; //multiplier
    var C = 1013904223; //increment

    var Random = function (_seed) {
        var seed;
        var z;

        var setSeed = function (newSeed) {
            seed = newSeed || Math.round(Math.random() * M); //get a random seed using Math.random();
            seed = seed % M;
            z = seed;
        };

        var getSeed = function () {
            return seed;
        };

        var getNext = function () {
            z = (A * z + C) % M;
            return z;
        };

        var getRandom = function () {
            return getNext() / M;
        };

        var getFloat = function (a, b) {
            if (!b) {
                b = a;
                a = 0;
            }
            if (!b) {
                b = 1;
            }

            return a + getRandom() * (b - a);
        };

        var getInt = function (a, b) {
            a = a || 0;
            b = b || 0;
            if (b < a) {
                var t = a;
                a = b;
                b = t;
            }
            return Math.floor(getFloat(a, b + 1));
        };

        var getBool = function (chance) {
            chance = chance === undefined ? 0.5 : chance;
            return (getRandom() < chance);
        };

        var pick = function (arr) {
            var idx = Math.floor(getRandom() * arr.length);
            return arr[idx];
        };

        //https://en.wikipedia.org/wiki/Marsaglia_polar_method
        var __ndSpare;
        var normalDist = function (mean, stdDev) {
            var u;
            var v;
            var s;
            if (__ndSpare !== undefined) {
                s = __ndSpare;
                __ndSpare = undefined;
                return mean + s * stdDev;
            }

            do {
                u = getRandom() * 2 - 1;
                v = getRandom() * 2 - 1;
                s = u * u + v * v
            } while (s > 1 || s === 0);
            var mul = Math.sqrt(-2 * Math.log(s) / s);
            __ndSpare = u * mul;
            return mean + v * mul * stdDev;
        };

        setSeed(_seed);

        var mod = {
            setSeed: setSeed,
            getSeed: getSeed,

            next: getNext,
            getNext: getNext,

            getRandom: getRandom,
            random: getRandom,

            getFloat: getFloat,
            float: getFloat,
            range: getFloat,

            getInt: getInt,
            int: getInt,

            getBool: getBool,
            boolean: getBool,

            pick: pick,

            normalDist: normalDist
        };

        Object.defineProperty(mod, "seed", {
            get: getSeed,
            set: setSeed
        });

        return mod;
    };

    Random.global = new Random();

    return Random;
});
/**
 * Module description
 */
bento.define('otemoto/utils/url', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/entity',
    'bento/eventsystem',
    'bento/utils',
    'bento/tween'
], function (
    Bento,
    Vector2,
    Rectangle,
    Entity,
    EventSystem,
    Utils,
    Tween
) {
    'use strict';
    var getParameter = function (key) {
        if (Utils.isCocoonJs()) {
            return undefined;
        }

        var url_string = window.location.href;
        var url = new URL(url_string);
        var val = url.searchParams.get(key);

        return val || undefined;
    };

    var module = {
        getParameter: getParameter
    };
    return module;
});
/**
 * Otemoto
 * @moduleName Otemoto
 */
bento.define('otemoto', [
    'otemoto/utils',
    'otemoto/modules/stylemanager',
    'otemoto/ui/label',
    'otemoto/ui/labelbutton',
    'otemoto/ui/dummy',
    'otemoto/ui/layout'
], function (
    Utils,
    StyleManager,
    Label,
    LabelButton,
    Dummy,
    Layout
) {
    return {
        Utils: Utils,
        modules: {
            StyleManager: StyleManager
        },
        ui: {
            Dummy: Dummy,
            Label: Label,
            LabelButton: LabelButton,
            Layout: Layout
        }
    };
});